# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging

from binascii import hexlify, unhexlify
from hashlib import pbkdf2_hmac
from os import urandom
from sqlalchemy import create_engine, Column, Integer, String, ForeignKey, UniqueConstraint
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, relationship
from sqlalchemy.orm.exc import NoResultFound

log = logging.getLogger (__name__)
Base = declarative_base ()

class PowerControl (Base):
    __tablename__ = 'powercontrol'

    id = Column (Integer, primary_key = True, autoincrement=True)

    URI      = Column (String, unique = True)
    rack_id  = Column (Integer, ForeignKey('rack.id'))
    rack     = relationship ("Rack", back_populates="powercontrol")
    ports    = relationship ("PowerPort", backref="powercontrol")
    model    = Column (String)
    __mapper_args__ = {'polymorphic_on': model}

    def __repr__ (self):
        return ("    Power[%s] %s" % (str(self.model), self.URI))

class PowerPort (Base):
    __tablename__ = 'powerport'

    id = Column (Integer, primary_key = True, autoincrement=True)
    powercontrol_id = Column (Integer, ForeignKey('powercontrol.id'))
    port = Column (Integer)

    def num_ports (self):
        return self.powercontrol.num_ports (self.port)
    def poweron (self):
        return self.powercontrol.poweron (self.port)
    def poweroff (self):
        return self.powercontrol.poweroff (self.port)
    def powerstatus (self):
        return self.powercontrol.powerstatus (self.port)

class SerialControl (Base):
    __tablename__ = 'serialcontrol'

    id = Column (Integer, primary_key = True, autoincrement=True)

    URI      = Column (String, unique = True)
    rack_id  = Column (Integer, ForeignKey('rack.id'))
    rack     = relationship ("Rack", back_populates="serialcontrol")
    ports    = relationship ('SerialPort', backref='serialcontrol')
    model    = Column (String)
    __mapper_args__ = {'polymorphic_on': model}

    def __repr__ (self):
        return ("    Serial[%s] %s" % (self.model, self.URI))

class SerialPort (Base):
    __tablename__ = 'serialport'

    id = Column (Integer, primary_key = True, autoincrement=True)

    serialcontrol_id = Column (Integer, ForeignKey ('serialcontrol.id'))
    port             = Column (Integer)
    udpport          = Column (Integer)

    def num_ports (self):
        self.control.num_ports (self.udpport)

class Slot (Base):
    __tablename__ = 'slot'

    id = Column (Integer, primary_key = True, autoincrement=True)

    position  = Column (Integer)
    rack_id   = Column (Integer, ForeignKey('rack.id'))
    board     = relationship ("Board", uselist=False, back_populates="slot")

    def __repr__ (self):
        return ("    ==SLOT[%d]== id<%d>" % (self.position, self.id))

class Rack (Base):
    __tablename__ = 'rack'

    id = Column (Integer, primary_key = True, autoincrement=True)

    name          = Column (String, unique = True)
    location      = Column (String)
    description   = Column (String)
    slots         = relationship ("Slot", backref = "rack")
    serialcontrol = relationship ("SerialControl", uselist=False, back_populates="rack")
    powercontrol  = relationship ("PowerControl", uselist=False, back_populates="rack")

    def __repr__ (self):
        desc = ("==Rack[%s]== id<%d>\n" % (self.name, self.id))
        if self.serialcontrol:
            desc += str(self.serialcontrol) + "\n"
        if self.powercontrol:
            desc += str(self.powercontrol)
        return desc

class Board (Base):
    __tablename__ = 'board'

    id = Column (Integer, primary_key = True, autoincrement=True)

    name    = Column (String, unique = True)
    URI     = Column (String)
    uuid    = Column (String (32), unique = True)
    slot_id = Column (Integer, ForeignKey ('slot.id'))
    slot    = relationship ("Slot", back_populates="board")

    def __repr__ (self):
        return ("        Board[%s] id<%d>" % (self.name, self.id))

class R4DDataBase (object):
    def __init__ (self, config):
        log.debug ("r4dd_db.__init__")

        self.__engine = create_engine (config.db_conf ('uri'),
                                       echo = config.db_conf_bool ('echo'))
        Base.metadata.create_all (self.__engine)
        self.__Session = sessionmaker (bind = self.__engine)

    def print_config (self):
        session = self.__Session ()
        racks = session.query(Rack)
        for r in racks:
            print (r)
            for s in r.slots:
                print (s)
                if (s.board):
                    print (s.board)
        session.close ()

    def get_session (self):
        return self.__Session ()

    def get_engine (self):
        return self.__engine



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging

from r4d.db import SerialControl

log = logging.getLogger (__name__)

def register (parent):
    log.debug ("register " + __name__)
    parent._add_model ('PS810', ps810)

class ps810 (SerialControl):
    __mapper_args__ = {'polymorphic_identity': 'PS810'}

    def num_ports (self):
        return 8

    def get_udpport (self, port):
        return (7000 + port)



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging
import sys

if sys.version_info >= (3,0):
    from http.server import HTTPServer
else:
    from BaseHTTPServer import HTTPServer

from pysimplesoap.server import SoapDispatcher, SOAPHandler, WSGISOAPHandler

log = logging.getLogger (__name__)

class R4DSoapService (object):
    def __init__(self, db):
        log.debug ("R4DSoapService.__init__")
        self.__dispatcher = SoapDispatcher (
            'r4d',
            location = "http://localhost:8008/",
            action = 'http://localhost:8008/',
            namespace = "http://ci-rt.linutronix.de/r4d.wsdl",
            prefix="r4d",
            pretty = True,
            debug = True)
        self.db = db

    def soap (self, f, name = None, returns = None, args = None, doc = None):
        if not name:
            name = self.__name__
        self.__dispatcher.register_function (name, f,
                                             returns = returns,
                                             args = args, doc = doc)

    def server (self, listen, port):
        httpd = HTTPServer ((listen, port), SOAPHandler)
        httpd.dispatcher = self.__dispatcher
        httpd.serve_forever ()



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2019  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os

from r4d.db import PowerControl
from snimpy.manager import Manager
from snimpy.manager import load

log = logging.getLogger (__name__)

mibpath = os.path.dirname (__file__)
load (os.path.join (mibpath, "GUDEADS-EPC8210-MIB.txt"))

def register (parent):
    log.debug ("register " + __name__)
    parent._add_model ('pc8210v4', pc8210)

class pc8210 (PowerControl):
    __mapper_args__ = {'polymorphic_identity': 'pc8210v4'}

    def num_ports (self):
        m = Manager (self.URI, "public", 2)
        return m.epc8210portNumber

    def poweron (self, port):
        m = Manager (self.URI, "private", 2)
        m.epc8210PortState [port] = 1

    def poweroff (self, port):
        m = Manager (self.URI, "private", 2)
        m.epc8210PortState [port] = 0

    def powerstatus (self, port):
        m = Manager (self.URI, "public", 2)
        return m.epc8210PortState [port]



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging
import r4d.serial

from r4d.db import SerialControl, SerialPort, Board, Rack
from pysimplesoap.server import SoapFault
from sqlalchemy.exc import IntegrityError
from r4d.tools import register_modules

log = logging.getLogger (__name__)

class SerialService (object):
    def __init__ (self, db, service):
        log.debug ('SerialService.__init__')

        # register callbacks
        if service:
            service.soap (self.get_serial,
                          'get-serial',
                          returns = {'host': str, 'port': int},
                          args = {'system': str})

        self.__db = db
        self.__models = {}
        register_modules (self, r4d.serial)

    def _add_model (self, model, f):
        self.__models.update ({model: f})

    def get_models (self):
        models = list (self.__models)
        return models

    def add_serial (self, model, host, _rack):
        """Add Serialcontrol"""
        try:
            M = self.__models [model]
        except Exception:
            log.error ("unknown model " + model)
            return -1


        session = self.__db.get_session ()
        session.begin (subtransactions=True)

        rack = session.query(Rack).filter(Rack.name == _rack).one ()
        if rack.serialcontrol:
            log.error (" a serialcontrol is already assigned to this rack")
            return -1

        serial = M (URI = host, rack_id = rack.id)
        session.add (serial)

        try:
            session.commit ()
        except IntegrityError as e:
            log.error (e)
            session.rollback ()
            session.close ()
            return -1

        for i in range (1, serial.num_ports () + 1):
            port = SerialPort (port = i, udpport = serial.get_udpport(i), serialcontrol_id = serial.id);
            session.add (port)

        try:
            session.commit ()
        except IntegrityError as e:
            log.error (e)
            session.rollback ()
            session.close ()
            return -1

        session.close ()
        return 0

    def get_serial (self, system):
        """Get Serialcontrol"""
        serport = {'host': 'none', 'port': -1}
        session = self.__db.get_session ()
        try:
            board = session.query (Board).filter_by (name = system).one ()
            serial = session.query (SerialControl).filter_by (rack_id = board.slot.rack_id).one ()
            port = session.query (SerialPort).filter_by (serialcontrol_id = serial.id, port = board.slot.position).one ()
            serport = {'host': serial.URI, 'port': port.udpport}
        except Exception as e:
            raise SoapFault (faultcode = "SerialGetSerialFailed",
                             faultstring = "get serial of {} failed".format (system),
                             detail = e)
        finally:
            session.close ()

        return serport

    def list_serial (self):
        """list Serialcontrols"""
        l = []
        session = self.__db.get_session ()
        try:
            for host in session.query (SerialControl):
               l.append ({'id': host.id,
                          'host': host.URI,
                          'model': host.model,
                          'ports': host.ports})
        except Exception as e:
            log.error ("serial control query failed: {}".format (e))
            return {'result': -1}
        return {'result': 0, 'list': l}



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging
import sys

from r4d.db import PowerControl

if sys.version_info >= (3,0):
    from html.parser import HTMLParser
    from urllib.request import urlopen
else:
    from HTMLParser import HTMLParser
    from urllib2 import urlopen

log = logging.getLogger (__name__)

def register (parent):
    log.debug ("register " + __name__)
    parent._add_model ('net8x', net8x)

class net8xHTMLParser(HTMLParser):
    old = ""
    status = {}

    def handle_data (self, data):
        d = {'ON': 1, 'OFF': 0}
        if data.find ("Power Port") >= 0:
            self.status [int (data.strip ()[-1:])] = d [self.old.strip ()]

        self.old = data

class net8x (PowerControl):
    __mapper_args__ = {'polymorphic_identity': 'net8x'}
    parser = net8xHTMLParser ()

    def __change (self, port, state):
        req = "http://{}/switch.html?cmd=1&p={}&s={}".format (self.URI,
                                                              port,
                                                              state)
        log.info (req)
        f = urlopen (req)
        log.info (f.read ())
        f.close ()

    def num_ports (self):
        return 8

    def poweron (self, port):
        self.__change (port, 1)

    def poweroff (self, port):
        self.__change (port, 0)

    def powerstatus (self, port):
        self.parser.status = {}

        f = urlopen ("http://" + self.URI)
        self.parser.feed (f.read ().decode ('utf-8'))
        f.close()

        try:
            return self.parser.status [port]
        except KeyError as e:
            return -1



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ELBE.  If not, see <http://www.gnu.org/licenses/>.

import logging

from r4d.config import R4DdConfig
from r4d.db import R4DDataBase
from r4d.soap import R4DSoapService
from r4d.rackservice import RackService
from r4d.powerservice import PowerService
from r4d.serialservice import SerialService
from r4d.boardservice import BoardService

log = logging.getLogger (__name__)

class R4Dd (object):
    def __init__ (self, configname):
        log.debug ("r4dd.__init__")
        self.__config = R4DdConfig (configname = configname)
        self.__db = R4DDataBase (self.__config)
        self.__service = R4DSoapService (self.__db)
        self.__power = PowerService (self.__db, self.__service)
        self.__rack = RackService (self.__db, self.__service)
        self.__serial = SerialService (self.__db, self.__service)
        self.__board = BoardService (self.__db, self.__service)

    def main (self):
        print ("Start Server...")

        listen = self.__config.r4dd_conf ("listen")
        port = int (self.__config.r4dd_conf ("port"))

        self.__service.server (listen, port)



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging

from r4d.db import Rack, Slot

from pysimplesoap.server import SoapFault

log = logging.getLogger (__name__)

class RackService (object):
    def __init__ (self, db, service):
        log.debug ('RackService.__init__')

        # register callbacks
        if service:
            service.soap (self.list_racks,
                          'list-racks',
                          returns = {'list': [{'name': str}]},
                          args = {})

        self.__db = db

    def add_rack (self, name, location, desc="", num_slots=8):
        session = self.__db.get_session ()
        try:
            rack = Rack (name=name, location=location, description=desc)
            for i in range(num_slots):
                slot = Slot (position=i+1)
                rack.slots.append (slot)
            session.add (rack)
            session.commit ()
        except Exception as e:
            log.error ("add new rack failed: {}".format (e))
            raise SoapFault (faultcode = "AddRackFailed",
                             faultstring = "could not add a new rack",
                             detail = e)
        finally:
            session.close ()

    def list_racks (self):
        """list Testracks"""
        l = []
        session = self.__db.get_session ()
        try:
            for rack in session.query (Rack):
                l.append ({'id': rack.id,
                           'name': rack.name,
                           'location': rack.location})

        except Exception as e:
            log.error ("testrack query failed: {}".format (e))
            raise SoapFault (faultcode = "ListSystemsFailed",
                             faultstring = "could not list testracks",
                             detail = e)
        finally:
            session.close ()

        log.info (l)
        return l



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import io
import logging
import sys

if sys.version_info >= (3,0):
    from configparser import RawConfigParser
else:
    from ConfigParser import RawConfigParser

log = logging.getLogger (__name__)

class PresetConfigParser (RawConfigParser):
    def __init__ (self, presets, **args):
        RawConfigParser.__init__(self, **args)
        for s in presets.keys():
            self.add_section (s)
            for k,v in presets[s].items():
                self.set (s,k,v)

class R4DdConfig (object):

    def __init__ (self, configname = None):
        log.debug ("r4dd_config.__init__")
        presets = { 'r4dd': {'listen': '0.0.0.0',
                                 'port': 8008},
                    'db':       {'echo': '0',
                                 'uri': 'sqlite:///r4d.sqlite',
                                 'user': None,
                                 'password': None} }

        self.__config = PresetConfigParser (presets, allow_no_value = True)
        if configname:
            self.load_config (configname)

    def load_config (self, name):
        self.__config.read (name)

    def r4dd_conf (self, name):
        return self.__config.get ("r4dd", name)

    def db_conf (self, name):
        return self.__config.get ("db", name)

    def db_conf_bool (self, name):
        return self.__config.getboolean ("db", name)



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging

from binascii import b2a_base64
from r4d.db import PowerPort, Slot, Rack, Board
from platform import node
from pysimplesoap.server import SoapFault
from sqlalchemy.exc import IntegrityError
from uuid import NAMESPACE_URL, UUID, uuid5

log = logging.getLogger (__name__)

class BoardService (object):
    def __init__ (self, db, service):
        log.debug ('BoardService.__init__')

        # register callbacks
        if service:
            service.soap (self.soap_list_boards,
                          'list-systems',
                          args = {'name': str},
                          returns = {'list': [{'location': str,
                                               'name': str,
                                               'port': int,
                                               'rack': str,
                                               'uuid': str}]})
        self.__db = db

    def add_board (self, name, rackname, slotnr):
        session = self.__db.get_session ()
        try:
            uri = 'r4d://' + node () + '/' + name

            rack = session.query (Rack).filter_by (name=rackname).one ()
            slot = session.query (Slot).filter_by (rack_id=rack.id, position=slotnr).one ()
            uuid = uuid5 (NAMESPACE_URL, str (uri))
            board = Board (name=name, slot_id=slot.id, URI=uri, uuid = uuid.hex)
            session.add (board)
            session.commit ()

        except IntegrityError as e:
            session.rollback ()
            log.error (e)

        session.close ()
        return 0

    def move_board (self, name, rackname, slotnr):
        session = self.__db.get_session ()
        try:
            board = session.query (Board).filter_by (name=name).one ()
            rack = session.query (Rack).filter_by (name=rackname).one ()
            dst_slot = session.query (Slot).filter_by (rack_id=rack.id, position=slotnr).one ()
            board.slot = dst_slot
            session.add (board)
            session.commit ()
        except IntegrityError as e:
            session.rollback ()
            log.error (e)
        session.close ()
        return 0

    def delete_board (self, name):
        session = self.__db.get_session ()
        try:
            board = session.query (Board).filter_by (name=name).one ()
            session.delete (board)
            session.commit ()
        except IntegrityError as e:
            session.rollback ()
            log.error (e)
        session.close ()
        return 0


    def list_boards (self, name = None):
        l = []
        session = self.__db.get_session ()
        try:
            q = session.query (Board)
            if name:
                q = q.filter_by (name = name)
            for board in q:
                uuid = UUID (hex = board.uuid)
                log.info (str(uuid))
                l.append ({'location': board.slot.rack.location,
                           'name': board.name,
                           'port': board.slot.position,
                           'rack': board.slot.rack.name,
                           'uuid': b2a_base64 (uuid.bytes_le)})
        except Exception as e:
            log.error ("board query failed: {}".format (e))
            return {'result': -1}
        finally:
            session.close ()
        return {'result': 0, 'list': l}

    def soap_list_boards (self, name):
        if name:
            l = self.list_boards (name = name)
        else:
            l = self.list_boards ()

        if l['result'] == -1:
            raise SoapFault (faultcode = "ListSystemsFailed",
                             faultstring = "could not list boards")

        return {'list': l['list']}



#!/usr/bin/env python3

from glob import glob
from setuptools import setup

from r4d.version import r4d_version

with open("README.rst") as f:
    long_description = f.read()

setup(name='r4d',
      version=r4d_version,
      description='R4D Daemon',
      long_description=long_description,
      author='Benedikt Spranger',
      author_email='b.spranger@linutronix.de',
      url='http://ci-rt.linutronix.de/',
      packages=['r4d', \
                'r4d.power', \
                'r4d.serial' ],
      package_data = {'r4d': ["power/*.txt"]},
      install_requires=['PySimpleSOAP',
                        'snimpy',
                        'SQLAlchemy' ],
      scripts=['r4dd', 'r4dcfg'],
      data_files= [
          ('/etc/',                glob("r4d.cfg")),
          ('/lib/systemd/system',  glob("r4dd.service"))],
      classifiers=[
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
      ],
)



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2016  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging
import r4d.power

from r4d.db import PowerControl, PowerPort, Board, Rack
from pysimplesoap.server import SoapFault
from sqlalchemy.exc import IntegrityError
from r4d.tools import register_modules

log = logging.getLogger (__name__)

class PowerService (object):
    def __init__ (self, db, service):
        log.debug ('PowerService.__init__')

        # register callbacks
        if service:
            service.soap (self.change,
                          'power-change',
                          args = {'system': str, 'state': bool},
                          returns = {'state': bool})

            service.soap (self.status,
                          'power-status',
                          args = {'system': str},
                          returns = {'state': bool})

        self.__db = db
        self.__models = {}
        register_modules (self, r4d.power)

    def _add_model (self, model, f):
        self.__models.update ({model: f})

    def get_models (self):
        models = list (self.__models)
        return models

    def add_power (self, model, host, _rack):
        """Add Powercontrol"""
        try:
            M = self.__models [model]
        except KeyError:
            log.error ("unknown model " + model)
            return -1

        session = self.__db.get_session ()
        session.begin (subtransactions=True)

        rack  = session.query(Rack).filter(Rack.name == _rack).one ()
        if rack.powercontrol:
            log.error (" a powercontrol is already assigned to this rack")
            return -1

        power = M (URI = host, rack_id = rack.id)
        session.add (power)

        try:
            session.commit ()
        except IntegrityError as e:
            log.error (e)
            session.rollback ()
            session.close ()
            return -1

        for i in range (1, power.num_ports () + 1):
            port = PowerPort (port = i, powercontrol_id = power.id);
            session.add (port)

        try:
            session.commit ()
        except IntegrityError as e:
            log.error (e)
            session.rollback ()
            session.close ()
            return -1

        session.close ()
        return 0

    def change (self, system, state):
        """Change Power Status"""
        session = self.__db.get_session ()
        try:
            board = session.query (Board).filter_by (name = system).one ()
            if board.slot:
                pos = board.slot.position
                if not board.slot or  not board.slot.rack or not board.slot.rack.powercontrol:
                        return {'state': False}
                pc_id = board.slot.rack.powercontrol.id
                powerport = session.query (PowerPort).filter_by (port = pos, powercontrol_id = pc_id).one ()
                if powerport:
                    if (int (state)):
                        powerport.poweron ()
                    else:
                        powerport.poweroff ()
                else:
                    return {'state': False}

        except Exception as e:
            log.error ("power: change status of {} failed: {}".format (system, e))
            raise SoapFault (faultcode = "PowerChangeFailed",
                             faultstring = "change status of {} failed".format (system),
                             detail = e)
        finally:
            session.close ()

        return {'state': True}

    def status (self, system):
        """Get Testsystem Power Status"""
        session = self.__db.get_session ()
        try:
            board = session.query (Board).filter_by (name = system).one ()
            if board.slot:
                pos = board.slot.position
                pc_id = board.slot.rack.powercontrol.id
                powerport = session.query (PowerPort).filter_by (port = pos, powercontrol_id = pc_id).one ()
                state = powerport.powerstatus ()
        except Exception as e:
            log.error ("power: get status of {} failed: {}".format (system, e))
            raise SoapFault (faultcode = "PowerStatusFailed",
                             faultstring = "get status of {} failed".format (system),
                             detail = e)
        finally:
            session.close ()
        return bool (state)

    def list_power (self):
        """list Powercontrols"""
        l = []
        session = self.__db.get_session ()

        try:
            for host in session.query (PowerControl):
                l.append ({'id': host.id,
                           'host': host.URI,
                           'model': host.model,
                           'ports': host.ports})
        except Exception as e:
            log.error ("power control query failed: {}".format (e))
            session.close ()
            return {'result': -1}

        session.close ()
        return {'result': 0, 'list': l}



# r4dd - rack infrastructure control daemon
#
# Copyright (C) 2019  Linutronix GmbH
# Author: Benedikt Spranger <b.spranger@linutronix.de>
#
# This file is part of r4d.
#
# r4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# r4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with r4d.  If not, see <http://www.gnu.org/licenses/>.

import logging
import time

from r4d.db import PowerControl

log = logging.getLogger (__name__)

def register (parent):
    log.debug ("register " + __name__)
    parent._add_model ('testbox', testbox)
    parent._add_model ('testbox-atx', testboxatx)

class testbox (PowerControl):
    __mapper_args__ = {'polymorphic_identity': 'testbox'}

    def num_ports (self):
        return 1

    def poweron (self, port):
        with open("/sys/class/leds/lamobo_r1:opto:powerswitch/brightness", "w") as gpio:
            gpio.write("255")

    def poweroff (self, port):
        with open("/sys/class/leds/lamobo_r1:opto:powerswitch/brightness", "w") as gpio:
            gpio.write("0")

    def powerstatus (self, port):
        with open("/sys/class/leds/lamobo_r1:opto:powerswitch/brightness", "r") as gpio:
            t = gpio.read()
            return int(t)

        return -1

class testboxatx (testbox):
    __mapper_args__ = {'polymorphic_identity': 'testbox-atx'}

    def poweron (self, port):
        super().poweron(port)
        with open("/sys/class/leds/lamobo_r1:opto:relay/brightness", "w") as gpio:
            time.sleep(10)
            gpio.write("255")
            gpio.seek(0)
            time.sleep(0.5)
            gpio.write("0")
